from typing import List, Dict, Any

from pydantic import validator

from virtool_core.models.searchresult import SearchResult
from virtool_core.models.basemodel import BaseModel


class HMMMinimal(BaseModel):
    id: str
    cluster: int
    count: int
    families: Dict[str, int]
    name: List[str]

    @validator("name")
    def is_name_valid(cls, name: List[str]) -> List[str]:
        if len(name) > 3:
            raise ValueError("The length of name should be a maximum of 3")
        return name


class HMMSequenceEntry(BaseModel):
    accession: str
    gi: str
    name: str
    organism: str


class HMM(HMMMinimal):
    entries: List[HMMSequenceEntry]
    genera: Dict[str, int]
    hidden: bool
    length: int
    mean_entropy: float
    total_entropy: float


class HMMSearchResult(SearchResult):
    documents: List[HMMMinimal]
    status: Dict[str, Any]
